# Workflow Scaffold

This directory is generated by `ztlctl workflow init` / `ztlctl workflow update`.

Selections:

- Source control: `{{ source_control }}`
- Profile: `{{ profile }}`
- Workflow mode: `{{ workflow }}`
- Skill set: `{{ skill_set }}`

Use `ztlctl workflow update` after upgrading ztlctl to refresh these generated notes.
